/*

    LED Ansteuerung für 8x WS2812B

*/

#include <Adafruit_NeoPixel.h>
#include "LED_Ringe.h"
#include "led_programme.h"

// IOs am Arduino
#define LED1_PIN 16
#define LED2_PIN 17
#define LED3_PIN 18
#define LED4_PIN 19
#define LED5_PIN 20
#define LED6_PIN 21
#define LED7_PIN 26
#define LED8_PIN 27

#define TASTE1_PIN 0
#define TASTE2_PIN 1
#define TASTE3_PIN 2
#define TASTE4_PIN 3

Adafruit_NeoPixel led1(LED_CNT, LED1_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led2(LED_CNT, LED2_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led3(LED_CNT, LED3_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led4(LED_CNT, LED4_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led5(LED_CNT, LED4_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led6(LED_CNT, LED4_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led7(LED_CNT, LED4_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel led8(LED_CNT, LED4_PIN, NEO_GRB + NEO_KHZ800);

uint16_t animation_time[9];
uint16_t animation_cnt[9];
bool led_refresh[9];
uint8_t led_mode[4];
bool led_mode_change[4];
const uint8_t tasten[4] = {TASTE1_PIN, TASTE2_PIN, TASTE3_PIN, TASTE4_PIN};

void setup(void) {
    led1.begin(); led1.clear(); led1.show();
    led2.begin(); led2.clear(); led2.show();
    led3.begin(); led3.clear(); led3.show();
    led4.begin(); led4.clear(); led4.show();
    led5.begin(); led5.clear(); led5.show();
    led6.begin(); led6.clear(); led6.show();
    led7.begin(); led7.clear(); led7.show();
    led8.begin(); led8.clear(); led8.show();

    pinMode(TASTE1_PIN, INPUT_PULLUP);
    pinMode(TASTE2_PIN, INPUT_PULLUP);
    pinMode(TASTE3_PIN, INPUT_PULLUP);
    pinMode(TASTE4_PIN, INPUT_PULLUP);
}

// Diverse Hilfsfunktionen

uint8_t *getLEDptr(uint8_t nr) {
  uint8_t *tmp = NULL;
  
  switch (nr) {
    case 1: tmp = led1.getPixels(); break;
    case 2: tmp = led2.getPixels(); break;
    case 3: tmp = led3.getPixels(); break;
    case 4: tmp = led4.getPixels(); break;
    case 5: tmp = led5.getPixels(); break;
    case 6: tmp = led6.getPixels(); break;
    case 7: tmp = led7.getPixels(); break;
    case 8: tmp = led8.getPixels(); break;
  }
  return tmp;
}

// Komplette LED-Kette einfarbig

void led_farbe(uint8_t nr, uint32_t color) {

  switch(nr) {
    case 1: led1.fill(color, 0, LED_CNT); break;
    case 2: led2.fill(color, 0, LED_CNT); break;
    case 3: led3.fill(color, 0, LED_CNT); break;
    case 4: led4.fill(color, 0, LED_CNT); break;
    case 5: led5.fill(color, 0, LED_CNT); break;
    case 6: led6.fill(color, 0, LED_CNT); break;
    case 7: led7.fill(color, 0, LED_CNT); break;
    case 8: led8.fill(color, 0, LED_CNT); break;
  }
  led_refresh[nr] = true;
}

/*
 * Konstantes Muster in LED-Kette laden
 */
void led_muster(uint8_t nr, const uint8_t *muster) {
  uint8_t *led;

  led = getLEDptr(nr);
  led_refresh[nr] = true;
  memcpy(led, muster, LED_CNT*3);
}

// LED-Muster links/rechts rotieren

void rotieren(uint8_t *led, bool dir_left) {
  uint8_t r,g,b;
  uint8_t *tmp;
  
  if (dir_left) {
    tmp = led;
    r = *tmp++;
    g = *tmp++;
    b = *tmp;
    memmove(led, led+3, (LED_CNT-1)*3);
    tmp = led + (LED_CNT-1)*3;
    *tmp++ = r;
    *tmp++ = g;
    *tmp   = b;
  } else {
    tmp = led + (LED_CNT-1)*3;
    r = *tmp++;
    g = *tmp++;
    b = *tmp;
    memmove(led+3, led, (LED_CNT-1)*3);
    tmp = led;
    *tmp++ = r;
    *tmp++ = g;
    *tmp   = b;
  }
}

// LED-Muster animieren durch rotieren

void animation_rotieren(uint8_t nr, uint16_t step, bool dir_left) {
  uint8_t *led;
  
  led = getLEDptr(nr);
  animation_time[nr] += step;
  if (animation_time[nr] >= 1000) {
    animation_time[nr] -= 1000;
    rotieren(led, dir_left);
    led_refresh[nr] = true;
  }
}

void animation_abspielen(uint8_t nr, uint8_t size, uint16_t step, const uint8_t *animation) {
  const uint8_t *tmp;
  uint8_t *led;
  
  led = getLEDptr(nr);
  animation_time[nr] += step;
  if (animation_time[nr] >= 1000) {
    animation_time[nr] -= 1000;    
    animation_cnt[nr]++;
    if (animation_cnt[nr] == size) animation_cnt[nr] = 0;   
    tmp = animation + (LED_CNT * 3 * animation_cnt[nr]);
    memcpy(led, tmp, LED_CNT*3); // Muster kopieren
    led_refresh[nr] = true;
  }  
}

void animation_reset(uint8_t nr) {
  animation_time[nr] = 0;
  animation_cnt[nr]  = 0;
  led_refresh[nr] = true;
}

// 4 Tasten lesen und Flanken erkennen, dabei Zähler hochzählen

void tasten_lesen(void) {
  static uint32_t last, now;
  static bool tasten_old[4];
  bool taste;
  
  now = millis();
  if ( (now - last) >= 100) {
    last = now;
    for (int i=0; i<4; i++) {
      led_mode_change[i] = false;
      taste = digitalRead(tasten[i]);
      if (taste == false && tasten_old[i] == true) {
        led_mode[i]++;
        if (led_mode[i] == MODE_CNT) led_mode[i] = 0;
        led_mode_change[i] = true;
      }
      tasten_old[i] = taste;
    }
  }
}

// Daten für LEDs neu übertragen

void leds_schreiben(void) {
  if (led_refresh[1]) led1.show();
  if (led_refresh[2]) led2.show();
  if (led_refresh[3]) led3.show();
  if (led_refresh[4]) led4.show();
  if (led_refresh[5]) led5.show();
  if (led_refresh[6]) led6.show();
  if (led_refresh[7]) led7.show();
  if (led_refresh[8]) led8.show();
  
  for (int i=1; i<9; i++) {
    led_refresh[i] = false;
  }
}

void loop (void) {
  static uint32_t last, now;
    
  tasten_lesen();
  now = millis();
  // Zeitraster für LED-Bearbeitung und Update
  if ( (now - last) >= TIME_STEP) {
    last += TIME_STEP;
    programe_aufrufen();
    leds_schreiben();      
  }
}
